'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var React = require('react');
var classnames = require('classnames');
var debounce = require('lodash/debounce');
var getOr = require('lodash/fp/getOr');
var utilsCommons = require('@highlight-ui/utils-commons');
var times = require('lodash/times');
var mapValues = require('lodash/fp/mapValues');

function _interopDefaultLegacy (e) { return e && typeof e === 'object' && 'default' in e ? e : { 'default': e }; }

var React__default = /*#__PURE__*/_interopDefaultLegacy(React);
var classnames__default = /*#__PURE__*/_interopDefaultLegacy(classnames);
var debounce__default = /*#__PURE__*/_interopDefaultLegacy(debounce);
var getOr__default = /*#__PURE__*/_interopDefaultLegacy(getOr);
var times__default = /*#__PURE__*/_interopDefaultLegacy(times);
var mapValues__default = /*#__PURE__*/_interopDefaultLegacy(mapValues);

/*! *****************************************************************************
Copyright (c) Microsoft Corporation.

Permission to use, copy, modify, and/or distribute this software for any
purpose with or without fee is hereby granted.

THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
PERFORMANCE OF THIS SOFTWARE.
***************************************************************************** */
/* global Reflect, Promise */

var extendStatics = function(d, b) {
    extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };
    return extendStatics(d, b);
};

function __extends(d, b) {
    if (typeof b !== "function" && b !== null)
        throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
    extendStatics(d, b);
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
}

var __assign = function() {
    __assign = Object.assign || function __assign(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};

var styles = {"table":"Table-module__5UQt0Iza__v3-0-1","fixedColumnCount1":"Table-module__23a3lOTU__v3-0-1","fixedColumnCount2":"Table-module__p_5q0eWA__v3-0-1","fixedColumnCount3":"Table-module__ZLjzqc4H__v3-0-1","fixedColumnCount4":"Table-module__1eRMj_cX__v3-0-1","fixedColumnCount5":"Table-module__1uMR-WlN__v3-0-1","fixedHeader":"Table-module__2HYMPw9Q__v3-0-1","defaultTheme":"Table-module__1UUtPtcW__v3-0-1","lightTheme":"Table-module__DrQCgXbg__v3-0-1"};

/* eslint-disable implicit-arrow-linebreak */
var getTableColumnWidth = function (tableElement, index) {
    var tableColumnElement = tableElement.querySelector("th:nth-child(" + (index + 1) + ")");
    return tableColumnElement
        ? tableColumnElement.getBoundingClientRect().width
        : 0;
};
var getTableColumnLeftOffset = function (columnWidths, index) {
    return columnWidths.slice(0, index).reduce(function (current, sum) { return sum + current; }, 0);
};
var getTableCells = function (tableElement, index) {
    return Array.from(tableElement.querySelectorAll("th:nth-child(" + (index + 1) + "), td:nth-child(" + (index + 1) + ")"));
};
var stackColumnsToTheLeft = function (tableElement, columnCount) {
    var columnWidths = times__default["default"](columnCount).map(function (index) {
        return getTableColumnWidth(tableElement, index);
    });
    var columnOffsets = columnWidths.map(function (_, index) {
        return getTableColumnLeftOffset(columnWidths, index);
    });
    columnOffsets.forEach(function (offset, index) {
        Array.from(getTableCells(tableElement, index)).forEach(function (cell) {
            // eslint-disable-next-line no-param-reassign
            cell.style.left = offset + "px";
        });
    });
};

var renderTh = function (props) {
    var column = props.column, metadata = props.metadata;
    return (React__default["default"].createElement("th", __assign({ key: column.key }, utilsCommons.parseMetadata(mapValues__default["default"](function (x) { return x + "-headCell"; }, metadata))), column.label));
};
var renderTd = function (props) {
    var column = props.column, content = props.content, metadata = props.metadata;
    return (React__default["default"].createElement("td", __assign({ key: column.key }, utilsCommons.parseMetadata(mapValues__default["default"](function (x) { return x + "-cell"; }, metadata))), content));
};
var renderTr = function (props) {
    var children = props.children, rowIndex = props.rowIndex, metadata = props.metadata;
    return (React__default["default"].createElement("tr", __assign({ key: rowIndex }, utilsCommons.parseMetadata(mapValues__default["default"](function (x) { return x + "-row"; }, metadata))), children));
};

var Table = /** @class */ (function (_super) {
    __extends(Table, _super);
    function Table(props) {
        var _this = _super.call(this, props) || this;
        _this.componentDidMount = function () {
            window.addEventListener('resize', _this.debouncedStackFixedColumns);
            _this.stackFixedColumns();
        };
        _this.componentDidUpdate = function () { return _this.debouncedStackFixedColumns(); };
        _this.componentWillUnmount = function () {
            window.removeEventListener('resize', _this.debouncedStackFixedColumns);
        };
        _this.stackFixedColumns = function () {
            var fixedColumnCount = _this.props.fixedColumnCount;
            var tableElement = _this.tableRef.current;
            if (!fixedColumnCount || tableElement == null)
                return;
            stackColumnsToTheLeft(tableElement, fixedColumnCount);
        };
        _this.tableRef = React__default["default"].createRef();
        _this.debouncedStackFixedColumns = debounce__default["default"](_this.stackFixedColumns, 200);
        return _this;
    }
    Table.prototype.render = function () {
        var _a = this.props, className = _a.className, _b = _a.columns, columns = _b === void 0 ? [] : _b, _c = _a.data, data = _c === void 0 ? [] : _c, _d = _a.fixedColumnCount, fixedColumnCount = _d === void 0 ? 0 : _d, fixedHeader = _a.fixedHeader, _e = _a.renderTd, renderTd$1 = _e === void 0 ? renderTd : _e, _f = _a.renderTh, renderTh$1 = _f === void 0 ? renderTh : _f, _g = _a.renderTr, renderTr$1 = _g === void 0 ? renderTr : _g, metadata = _a.metadata, _h = _a.theme, theme = _h === void 0 ? 'default' : _h;
        var innerMetadata = utilsCommons.suffixMetadata(metadata);
        return (React__default["default"].createElement("table", __assign({ className: classnames__default["default"](className, fixedHeader ? styles.fixedHeader : null, styles.table, styles["fixedColumnCount" + fixedColumnCount], theme ? styles[theme + "Theme"] : styles.defaultTheme), ref: this.tableRef }, utilsCommons.parseMetadata(metadata)),
            React__default["default"].createElement("thead", __assign({}, utilsCommons.parseMetadata(innerMetadata('head'))),
                React__default["default"].createElement("tr", __assign({}, utilsCommons.parseMetadata(innerMetadata('headRow'))), columns
                    .map(function (column) { return (__assign({ renderTh: renderTh$1 }, column)); })
                    .map(function (column, columnIndex) {
                    return column.renderTh({ column: column, columnIndex: columnIndex, metadata: metadata });
                }))),
            React__default["default"].createElement("tbody", __assign({}, utilsCommons.parseMetadata(innerMetadata('body'))), data.map(function (dataRow, rowIndex) {
                return renderTr$1({
                    children: columns
                        .map(function (column) { return (__assign({ renderTd: renderTd$1 }, column)); })
                        .map(function (column, columnIndex) {
                        return column.renderTd({
                            column: column,
                            columnIndex: columnIndex,
                            content: getOr__default["default"](undefined, column.key || '', dataRow),
                            dataRow: dataRow,
                            rowIndex: rowIndex,
                            metadata: metadata,
                        });
                    }),
                    dataRow: dataRow,
                    rowIndex: rowIndex,
                    metadata: metadata,
                });
            }))));
    };
    return Table;
}(React__default["default"].Component));

exports.Table = Table;
//# sourceMappingURL=index.js.map
